{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "sh",
  "clipboard.copy": "Kopiraj u klipbord",
  "clipboard.copied": "Iskopirano u klipbord",
  "edit.link.title": "Ažuriraj stranicu",
  "footer.previous": "Prethodno",
  "footer.next": "Sledeće",
  "footer.title": "Podnožje",
  "header.title": "Zaglavlje",
  "meta.comments": "Komentari",
  "meta.source": "Izvor",
  "nav.title": "Navigacija",
  "search.placeholder": "Pretraga",
  "search.share": "Deljenje",
  "search.reset": "Očisti",
  "search.result.initializer": "Inicijalizujem pretragu",
  "search.result.placeholder": "Unesite pojam pretrage",
  "search.result.none": "Ništa nije pronađeno",
  "search.result.one": "1 rezultat pretrage",
  "search.result.other": "# rezultata pretrage",
  "search.result.more.one": "još 1 na ovoj strani",
  "search.result.more.other": "još # na ovoj strani",
  "search.result.term.missing": "Nedostaje",
  "search.title": "Pretraga",
  "select.language.title": "Izaberi jezik",
  "select.version.title": "Izaberi verziju",
  "skip.link.title": "Idi na tekst",
  "source.link.title": "Idi u repozitorijum",
  "source.file.date.updated": "Ažuriran",
  "source.file.date.created": "Kreiran",
  "tabs.title": "Tabovi",
  "toc.title": "Sadržaj",
  "top.title": "Nazad na vrh"
}[key] }}{% endmacro %}
