////
/// Copyright (c) 2016-2022 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Back-to-top button
.md-top {
  position: fixed;
  top: px2rem(48px + 16px);
  z-index: 2;
  margin-inline-start: 50%;
  padding: px2rem(8px) px2rem(16px);
  color: var(--md-default-fg-color--light);
  font-size: px2rem(14px);
  background-color: var(--md-default-bg-color);
  border-radius: px2rem(32px);
  outline: none;
  box-shadow: var(--md-shadow-z2);
  transform: translate(-50%, 0);
  transition:
    color            125ms,
    background-color 125ms,
    transform        125ms cubic-bezier(0.4, 0, 0.2, 1),
    opacity          125ms;

  // [print]: Hide back-to-top button
  @media print {
    display: none;
  }

  // Adjust for right-to-left languages
  [dir="rtl"] & {
    transform: translate(50%, 0);
  }

  // Back-to-top button in hidden state
  &[data-md-state="hidden"] {
    transform: translate(-50%, px2rem(4px));
    opacity: 0;
    transition-duration: 0ms;
    pointer-events: none;

    // Adjust for right-to-left languages
    [dir="rtl"] & {
      transform: translate(50%, px2rem(4px));
    }
  }

  // Back-to-top button on focus/hover
  &:is(:focus, :hover) {
    color: var(--md-accent-bg-color);
    background-color: var(--md-accent-fg-color);
  }

  // Inline icon
  svg {
    display: inline-block;
    vertical-align: -0.5em;
  }
}
