{#-
  This file was automatically generated - do not edit
-#}
{% if meta and 'hero' in meta %}
{% set hero = meta['hero'] | e %}
<div class="md-hero{% if nav_bar_tabs %} md-hero--expand{% endif %}" data-md-component="hero">
  <div class="md-hero__inner md-grid">
    <p>{{ hero }}</p>
  </div>
</div>
{% endif %}
